//********************************************************************
//      Copyright ©2014-2023 Gowin Semiconductor Corporation.
//                      All rights reserved.
//
// <File>     : gw_ahb_ram.v
// <Author>   : EMB
// <Function> : AHB RAM
// <Version>  : 1.0
// <Date>     : 2023
//
//********************************************************************

// gw_ahb_ram
module gw_ahb_ram
(
    output  reg     [31:0]  AHB_HRDATA,
    output  wire            AHB_HREADYOUT,
    output  wire    [ 1:0]  AHB_HRESP,
    input   wire    [ 1:0]  AHB_HTRANS,
    input   wire    [ 2:0]  AHB_HBURST,
    input   wire    [ 3:0]  AHB_HPROT,
    input   wire    [ 2:0]  AHB_HSIZE,
    input   wire            AHB_HWRITE,
    input   wire    [31:0]  AHB_HADDR,
    input   wire    [31:0]  AHB_HWDATA,
    input   wire            AHB_HSEL,
    input   wire            AHB_HCLK,
    input   wire            AHB_HRESETn
);

parameter  ADDR_WIDTH    = 7 ;  // Address is 128, bit width is 7
parameter  DATA_WIDTH    = 16;  // Data bit width

assign AHB_HREADYOUT = 1'b1;    // Ready signal, slave to MCU master
//Response OKAY
assign AHB_HRESP  = 2'b0;       // Response signal, slave to MCU master

// Define Register for AHB BUS
reg [31:0]  ahb_address;
reg         ahb_control;
reg         ahb_sel;
reg         ahb_htrans;

reg  [DATA_WIDTH-1:0]  din;
reg  [DATA_WIDTH-1:0]  dout;
reg  [1:0]             cmd;     // sel 2'b10 ,wrd 2'01
reg  [ADDR_WIDTH-1:0]  addr;

reg  sel;
reg  wrd;
wire [DATA_WIDTH-1:0]  d_out;

always @(posedge AHB_HCLK)
begin
    if(~AHB_HRESETn)
    begin
        ahb_address  <= 32'h0;
        ahb_control  <= 1'b0;
        ahb_sel      <= 1'b0;
        ahb_htrans   <= 1'b0;
    end
    else
    begin
        ahb_address  <= AHB_HADDR;
        ahb_control  <= AHB_HWRITE;
        ahb_sel      <= AHB_HSEL;
        ahb_htrans   <= AHB_HTRANS[1];
    end
end

wire write_enable = ahb_htrans & ahb_control    & ahb_sel;
wire read_enable  = ahb_htrans & (!ahb_control) & ahb_sel;

// Write data to AHB bus
always @(posedge AHB_HCLK)
begin
    if(~AHB_HRESETn)
    begin
        cmd  <= 2'b0;
        addr <= 7'b0;
        din  <= 16'b0;
    end
    else if(ahb_control)
    begin
        case (ahb_address[15:0])
        16'h0000: cmd  <= AHB_HWDATA[1:0];
        16'h0004: addr <= AHB_HWDATA[ADDR_WIDTH-1:0];
        16'h0008: din  <= AHB_HWDATA[DATA_WIDTH-1:0];
        endcase
    end
end

always @(posedge AHB_HCLK)
begin
    if(~AHB_HRESETn)
    begin
        sel <= 1'b0;
        wrd <= 1'b0;
    end
    else    // if(write_enable)
    begin
        sel <= cmd[1];
        wrd <= cmd[0];
    end
end

// Read data to AHB bus 
always @(posedge AHB_HCLK)
begin
    if(~AHB_HRESETn)
    begin
        dout <= 16'b0;
    end
    else
    begin
        dout <= d_out;
    end
end

// Register address
reg [31:0] ahb_rdata;

always @(*)
begin
    if(!ahb_control)  // Read cmd
    begin
        case (ahb_address[15:0])
        16'h0000:  ahb_rdata = cmd;		
        16'h0004:  ahb_rdata = addr;
        16'h0008:  ahb_rdata = din;
        16'h000C:  ahb_rdata = dout;
        default:   ahb_rdata = 32'hFFFFFFFF;
        endcase
    end
    else
    begin
        ahb_rdata = 32'hFFFFFFFF;
    end
end

assign AHB_HRDATA = ahb_rdata;

reg  oce = 1'b1;

// Gowin_DPB instantiation
Gowin_DPB u_Gowin_DPB
(
    .douta(d_out),
    .doutb(),
    .clka(AHB_HCLK),
    .ocea(oce),
    .cea(sel),
    .reseta(~AHB_HRESETn),
    .wrea(wrd),
    .clkb(),
    .oceb(),
    .ceb(),
    .resetb(),
    .wreb(),
    .ada(addr),
    .dina(din),
    .adb(),
    .dinb()
);

endmodule